--[[
    Torta: A World of Warcraft addon that helps you keep track of things
    Copyright (C) 2008-2009 Lawrence Patrick Calulo (Quezacolt)

    Torta is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Torta is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Torta.  If not, see <http://www.gnu.org/licenses/>.
]]

-- ====================================================================
--  DnD.lua
--  Stands for "Dropdowns and Dialogs", not "Do not Disturb" or
--  "Dungeons and Dragons" :)
--  Contains code for dropdowns, dialogs, and anything related to them
-- ====================================================================

Torta:RegisterRevision("$Revision: 30 $");

-- Capitalization rules:
-- http://java.sun.com/products/jlf/ed2/book/HIG.Visual3.html

local clExplorerDropdownInfo = {
  {
    text = "Tortoise Explorer",
    isTitle = true
  },
  {
    text = "Create a New List",
    func = function() Torta.cl3:CreateList(); end
  },
  {
    text = "Manage Profiles",
    func = function() LibStub("AceConfigDialog-3.0"):Open("Torta"); end
  },
  {
    text = "",
    notClickable = true
  },
  {
    text = "Close",
    func = function() TortaDropDownMenuFrame:Hide(); end
  }
}

local clDispWndDropdownInfo = {
  {
    text = "Torta Checklist",
    isTitle = true
  },
  --[[
  {
    text = "Add Timers and Alerts...",
    func = function() Torta.alerts:OpenWindow(); end
  },
  --]]
  {
    text = "Create a New Task",
    func = function() Torta.cl3:AddTask(); end
  },
  {
    text = "Manage Profiles",
    func = function() LibStub("AceConfigDialog-3.0"):Open("Torta"); end
  },
  {
    text = "",
    notClickable = true
  },
  {
    text = "Close",
    func = function() TortaDropDownMenuFrame:Hide(); end
  },
}

local clListDropdownInfo = {
  {
    text = "<will get replaced by list name>",
    isTitle = true
  },
  {
    text = "Load",
    func = function() Torta.cl3:LoadList(Torta.gui.selectedExplorerButton:GetID()); end
  },
  {
    text = "Rename",
    func = function() Torta.cl3:RenameList(Torta.gui.selectedExplorerButton:GetID()); end
  },
  {
    text = "Delete",
    func = function() Torta.cl3:DeleteList(Torta.gui.selectedExplorerButton:GetID()); end
  },
  {
    text = "",
    notClickable = true
  },
  {
    text = "Create a New List",
    func = function() Torta.cl3:CreateList(); end,
  },
  {
    text = "Manage Profiles",
    func = function() LibStub("AceConfigDialog-3.0"):Open("Torta"); end
  },
  {
    text = "",
    notClickable = true
  },
  {
    text = "Close",
    func = function() TortaDropDownMenuFrame:Hide(); end
  }
}

-- For the dropdown when you right-click a task

local clTaskDropdownInfo = {
  {
    text = "Task *n*",
    isTitle = true
  },
  {
    text = "Remove",
    func = function() Torta.cl3:DeleteTaskByID(Torta.gui.selectedTaskID); end
  },
  {
    text = "Create a new task below this",
    func = function() Torta.cl3:AddTask(Torta.gui.selectedTaskID + 1); end
  },
  {
    text = "Edit",
    func = function() Torta.cl3:EditTask(Torta.gui.selectedTaskID); end
  },
  {
    text = "",
    notClickable = true
  },
  --[[
  {
    text = "Add Alerts...",
    func = function() Torta.alerts:OpenWindow(); end
  },
  --]]
  {
    text = "Close",
    func = function() TortaDropDownMenuFrame:Hide(); end
  }
}

-- For Notebook

local nbExplorerDropdownInfo = {
  {
    text = "Tortoise Explorer",
    isTitle = true
  },
  {
    text = "Create a New Note",
    func = function()  end
  },
  {
    text = "Manage Profiles",
    func = function() LibStub("AceConfigDialog-3.0"):Open("Torta"); end
  },
  {
    text = "",
    notClickable = true
  },
  {
    text = "Close",
    func = function() TortaDropDownMenuFrame:Hide(); end
  }
}

local dropdown = CreateFrame("Frame", "TortaDropDownMenuFrame", explorerWindow, "UIDropDownMenuTemplate");

local function showMenu(menu)
  EasyMenu(menu, dropdown, "cursor", 0, 0, "MENU", 3);
end

-- Checklist

function Torta.gui:clShowButtonDropdown(button)
  assert(button);
  self.selectedExplorerButton = button;
  clListDropdownInfo[1].text = button.text:GetText();
  if Torta.cl3:GetList(button:GetID()) == Torta.cl3:GetLoadedList() then
    clListDropdownInfo[2].text = "Unload";
  else
    clListDropdownInfo[2].text = "Load";
  end
  showMenu(clListDropdownInfo);
end

function Torta.gui:clShowExplorerDropdownMenu()
  showMenu(clExplorerDropdownInfo);
end

function Torta.gui:clShowDispWndDropdownMenu()
  clDispWndDropdownInfo[1].text = Torta.cl3:GetLoadedList().meta.listName;
  showMenu(clDispWndDropdownInfo);
end

function Torta.gui:clShowTaskDropdown(button)
  local id = button:GetID();
  self.selectedTaskID = id;
  clTaskDropdownInfo[1].text = "Task " .. id;
  showMenu(clTaskDropdownInfo);
end

-- Notebook

function Torta.gui:nbShowExplorerDropdownMenu()
  showMenu(nbExplorerDropdownInfo);
end

-- ====================================================================
--  Dialogs
-- ====================================================================
StaticPopupDialogs["TORTA_CLRENAMELIST"] = {
  text = "Please enter the new name for the list currently named \"|cffffd000%s|r\":",
  OnAccept = function(frame)
    local newName = getglobal(frame:GetName() .. "EditBox"):GetText();
    Torta.cl3:RenameList(Torta.gui.selectedExplorerButton:GetID(), newName);
    frame:Hide();
  end,
  EditBoxOnEnterPressed = function(editBox)
    local newName = editBox:GetText();
    Torta.cl3:RenameList(Torta.gui.selectedExplorerButton:GetID(), newName);
    editBox:GetParent():Hide();
  end,
  EditBoxOnEscapePressed = function(self)
    self:GetParent():Hide();
  end,
  button1 = ACCEPT,
  button2 = CANCEL,
  whileDead = 1,
  timeout = 30,
  hasEditBox = 1,
  hideOnEscape = 1,
}
StaticPopupDialogs["TORTA_CLDELETELIST"] = {
  text = "Are you ABSOLUTELY sure you want to delete the list \"|cffffd000%s|r\"?",
  button1 = YES,
  button2 = NO,
  showAlert = true,
  OnAccept = function(self, data)
    Torta.cl3:DeleteList(data, true);
  end,
  whileDead = 1,
  timeout = 30,
  hideOnEscape = 1
}
