--[[
    Torta: A World of Warcraft addon that helps you keep track of things
    Copyright (C) 2008-2009 Lawrence Patrick Calulo (Quezacolt)

    Torta is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Torta is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Torta.  If not, see <http://www.gnu.org/licenses/>.
]]

-- ====================================================================
--  LDB.lua
--  Contains code for LDB. Crazy, I know.
-- ====================================================================

local ldb = LibStub("LibDataBroker-1.1");

local dataobj = ldb:NewDataObject("Torta Checklist", {
  type = "data source",
  text = "Torta Checklist",
  icon = "Interface\\FriendsFrame\\FriendsFrameScrollIcon",
  OnClick = function(frame, button)
    if button == "LeftButton" then
      Torta.gui:ToggleWindow();
    else
      -- Put a dropdown menu here later?
    end
  end
});

local stuffToDisplay = {};
function dataobj:OnTooltipShow()
  local list = Torta.cl3:GetLoadedList();
  local tasksToDoOver10 = false;
  
  table.wipe(stuffToDisplay)
  
  if list then -- We'll throw errors if we don't have a list loaded and attempt to make a data-filled tooltip.
    for i = 1, #list.text do
      if not list.stat[i] then
        tinsert(stuffToDisplay, list.text[i]);
        if #stuffToDisplay >= 10 then
          tasksToDoOver10 = true;
          break;
        end
      end
    end
    
    self:AddLine("Torta Checklist: |cffffffff" .. list.meta.listName);
    if #stuffToDisplay == 0 then
      self:AddLine("You're all done! Hooray!", 1, 1, 1);
    elseif not tasksToDoOver10 then
      self:AddLine("Unfinished Tasks", 1, 1, 1);
    else
      self:AddLine("Unfinished Tasks (First ten)", 1, 1, 1);
    end
    
    for i,v in ipairs(stuffToDisplay) do
      self:AddLine(i .. ".) " .. v, 1, 1, 1, true);
    end
  else -- If we don't have a list loaded right now...
    self:AddLine("Torta Checklist");
    self:AddLine("No list is currently loaded.", 1, 1, 1);
  end
end

function dataobj:OnEnter()
  GameTooltip:SetOwner(self, "ANCHOR_NONE");
  GameTooltip:SetPoint("TOPLEFT", self, "BOTTOMLEFT");
  GameTooltip:ClearLines();
  dataobj.OnTooltipShow(GameTooltip);
  GameTooltip:Show();
end

function dataobj:OnLeave()
  GameTooltip:Hide();
end

-- LDB update function (duh!)
function Torta.gui:UpdateLDB()
  local newText;
  local list = Torta.cl3:GetLoadedList();
  if list then
    local checked, total = Torta.cl3:GetListStats();
    dataobj.text = list.meta.listName .. " (|cffffffff" .. checked .. "|r/|cffffffff" .. total .. "|r)";
  else
    dataobj.text = "Torta Checklist";
  end
end
