--[[
    Torta: A World of Warcraft addon that helps you keep track of things
    Copyright (C) 2008-2009 Lawrence Patrick Calulo (Quezacolt)

    Torta is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Torta is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Torta.  If not, see <http://www.gnu.org/licenses/>.
]]
-- ====================================================================
--  UIManager.lua
--  Contains functions used for operations that impact what the user
--  sees.
-- ====================================================================

-- Do you seriously expect me to type the entirety of these out every single
-- time I need to use them?

-- Windows
local          mainWindow = TortaParentFrame;
local           startPage = TortaParentFrameStartPage;
local            mainPage = TortaParentFrameMainPage;
local      explorerWindow = TortaParentFrameMainPageExplorerSubWindow;
local     clDisplayWindow = TortaParentFrameMainPageCLDisplaySubWindow;
local     nbDisplayWindow = TortaParentFrameMainPageNBDisplaySubWindow;

-- Misc. Widgets
local       exScrollFrame = TortaParentFrameMainPageExplorerSubWindowScrollBar;
local         clScrollBar = TortaParentFrameMainPageCLDisplaySubWindowScrollBar;
local    noLoadedListText = TortaParentFrameMainPageCLDisplaySubWindowNoLoadedListText;
local         noTasksText = TortaParentFrameMainPageCLDisplaySubWindowNoTasksDetailsText;
local         clTopButton = TortaParentFrameMainPageCLDisplaySubWindowTaskButton1;
local       cl3HeaderText = TortaParentFrameMainPageCLDisplaySubWindowHeaderText;
local          taskEditor = TortaParentFrameMainPageTaskEditorSubWindow;
local        alertsEditor = TortaTimersAndAlertsFrame;
local           clTaskBar = TortaParentFrameMainPageCLTaskbar;
local           nbTaskBar = TortaParentFrameMainPageNBTaskbar;
local          editTaskEB = TortaParentFrameMainPageTaskEditorSubWindowScrollFrameEditBox;
local    uncheckAllButton = TortaParentFrameMainPageCLTaskbarUncheckAllButton;
local      checkAllButton = TortaParentFrameMainPageCLTaskbarCheckAllButton;
local       addTaskButton = TortaParentFrameMainPageCLTaskbarAddTaskButton;
local    removeTaskButton = TortaParentFrameMainPageCLTaskbarRemoveTaskButton;
local explorerNoListsText = TortaParentFrameMainPageExplorerSubWindowNoListsText;
local explorerNoNtbksText = TortaParentFrameMainPageExplorerSubWindowNoNotebooksText;
local      exBadRegexText = TortaParentFrameMainPageExplorerSubWindowBadRegexText;
local         statsButton = TortaParentFrameMainPageCLTaskbarStatsButton;
local         welcomeText = TortaParentFrameStartPageWelcomeSubWindowScrollFrameContentsText;

-- ====================================================================
--  Allow public access to our "shortcuts"
-- ====================================================================
Torta.gui.elements = {};
local e = Torta.gui.elements;
e.mainWindow          = mainWindow;
e.startPage           = startPage;
e.explorerWindow      = explorerWindow;
e.exScrollFrame       = exScrollFrame;
e.clDisplayWindow     = clDisplayWindow;
e.clScrollBar         = clScrollBar;
e.noLoadedListText    = noLoadedListText;
e.clTopButton         = clTopButton;
e.cl3HeaderText       = cl3HeaderText;
e.taskEditor          = taskEditor;
e.alertsEditor        = alertsEditor;
e.clTaskBar           = clTaskBar;
e.nbTaskBar           = nbTaskBar;
e.editTaskEB          = editTaskEB;
e.uncheckAllButton    = uncheckAllButton;
e.checkAllButton      = checkAllButton;
e.addTaskButton       = addTaskButton;
e.removeTaskButton    = removeTaskButton;
e.explorerNoListsText = explorerNoListsText;
e.explorerNoNtbksText = explorerNoNtbksText;
e.exBadRegexText      = exBadRegexText;
e.statsButton         = statsButton;

-- Specifically required by UIUpdate.lua
e.noTasksText         = noTasksText;
e.nbDisplayWindow     = nbDisplayWindow;
e.noLoadedListTest    = noLoadedListText;


e = nil;
-- ====================================================================

-- Etc.
local EXPLORER_LISTBUTTON_HEIGHT = 16;
local Torta = Torta;
Torta.gui.canReorderLists = true;

noLoadedListText:SetText("|cffffd000What? No loaded list? I... I feel ignored :(|r\nYou currently do not have a list loaded. You can either select a list in Tortoise Explorer or create a new one by right-clicking inside Tortoise Explorer.");
noTasksText:SetText("|cffffd000Woah, no tasks!|r\nThe currently loaded list does not appear to have any tasks in it. You can create new tasks for this list by clicking the \"Add A Task\" button below, or right click here to open a dropdown menu.");

TortaParentFrameStartPageSidebarAboutText:SetText("Torta is an addon written by Quezacolt of Aman'Thul based on the Checklist project.\n\nYour suggestions for Torta are always welcome :)");
TortaParentFrameStartPageSidebarAboutText:SetJustifyV("TOP");

--[[
Notes to self:

  * Make the URLs a hyperlink that pops up a dialog with an editbox containing the URL.
    Use Prat's functionality if it's available.
    
  * Switch from using a FontString to SimpleHTML.
--]]

-- Note: the "If you experience trouble..." paragraph is a subtle way of saying "I'd prefer it if you didn't download this through WoWMatrix" XD
welcomeText:SetText([[
Thank you for downloading Torta! If you have any problems while using this addon and the solution is not in this page, you can find additional support at either of the following sites:

http://www.wowinterface.com/downloads/info11619-Torta.html
http://wow.curse.com/downloads/wow-addons/details/torta.aspx
http://www.wowace.com/projects/torta/

If you experience trouble with a modified version of Torta and your copy is from somewhere other than Curse, WoWInterface or WoWAce, try the latest version from one of the sites listed above.

Torta is licensed under the GNU General Public License version 3, which makes it |cffffd000free and open-source software|r. It is free in the sense of both "free beer" and "free speech". The full license text can be found in LICENSE.txt in this addon's folder.

|cffffd000The Torta User's Manual|r

|cffffd000Table of Contents|r
|cffffd0001.|r What's new in Torta?
|cffffd0002.|r What's in a name?
|cffffd0003.|r Torta Checklist
  |cffffd0003.1.|r Timers and Alerts
|cffffd0004.|r Torta Notebook
|cffffd0005.|r Quirks


|cffffd000Section 1. What's new in Torta?|r
Torta is the next generation of the Checklist addon, keeping most of the latter's features while providing a solid platform for additional ones.

Under the hood, Torta features a re-designed database system which takes a smaller memory footprint and is easier to work with for the developer.

With these improvements, it has become much easier to extend the addon to do what users want it to do. The code is more organized, allowing for easier debugging and greater stability. 


|cffffd000Section 2. What's in a name?|r
Torta was picked to be the name for this addon because:

Torta is the name of a tortoise in Tanaris. Tortoises are tough. Torta is tough. Unless I messed up someplace.

The SVN client that I use is TortoiseSVN. More tortoise insanity.

In the Philippines, 'torta' refers to a kind of omelette made with eggs, ground meat and sometimes minced onion and potato. (See http://en.wikipedia.org/wiki/Torta)

Lastly, the name "Checklist" no longer fit the addon, as it left out the notes module added in the latter part of Checklist2's lifespan. "Torta" does not describe any one part of the addon, leaving it to be free to expand, contract, bounce, or cause some other equal and opposite reaction as described in Newton's third law of motion.


|cffffd000Section 3. Torta Checklist|r
Torta Checklist was created with the intent of helping you keep track of the various things you have to routinely do or do in a specific order (or both!).

Lists in Turtle Explorer and tasks in the Torta Checklist window are drag-and-drop reorderable. It's fun to simply drag them around, too! 


|cffffd000Section 3.1. Timers and Alerts|r



|cffffd000Section 4. Torta Notebook|r
This section will be filled in as Torta Notebook is worked on. Update Torta every week or so from WoWInterface or Curse to get the latest updates.

|cffffd000Section 5. Quirks|r

|cffffd000"Why does this help text vanish when the window is scaling in/scaling out?"|r
When this window scales in/out with this text visible, WoW constantly recalculates stuff concerning the size of this text, the size of the scroll frame it's in, and maybe other stuff. Because this text is pretty damn long, this causes quite a performance hit, enough to make me hide the text before any re-scaling is done.

|cffffd000"Why is the shortcut to saving a task shift-ESC? You should just use the normal way of doing things like CTRL-Enter!"|r
When set to be multi-line, WoW's editbox widget does not tell the addon when enter has been pressed. I used a multi-line editbox so more text can be visible at once, unlike in earlier versions of Checklist.

]]);
welcomeText:SetAlpha(0);
welcomeText:Hide();

-- Explorer subwindow stuff
Torta.gui.exRecycleBin = {};
Torta.gui.exListButtons = { TortaParentFrameMainPageExplorerSubWindowListButton1 };
-- Checklist subwindow stuff
Torta.gui.clRecycleBin = {};
Torta.gui.clListButtons = { TortaParentFrameMainPageCLDisplaySubWindowTaskButton1 };


function Torta.gui.scalingFinishedFunc(frame)
  frame:Hide();
end

local FADETIME = 0.25;
function Torta.gui:ToggleWindow(window)
  window = window or mainWindow;
  if self:IsFrameScaling(window) or UIFrameIsFading(window) then
    return;
  end
  if not window:IsVisible() then
    window.scalingFinishedFunc = nil;
    window:Show();
    self:ScaleIn(window, FADETIME, 0.85, 1);
    UIFrameFadeIn(window, FADETIME, 0, 1);
  else
    if window == mainWindow then
      welcomeText:Hide();
    end
    window.scalingFinishedFunc = self.scalingFinishedFunc;
    self:ScaleOut(window, FADETIME, 1, 0.85);
    UIFrameFadeOut(window, FADETIME, 1, 0);
  end
end


function Torta.gui:OnTabClicked(tabID)
  assert(type(tabID) == "number");
  PanelTemplates_SetTab(mainWindow, tabID);
  if Torta.cl3.HideTaskEditor then
    Torta.cl3:HideTaskEditor();
  end
  if tabID == 1 then
    startPage:Show();
    mainPage:Hide();
  else
    self.tab = tabID;
    startPage:Hide();
    mainPage:Show();
    self:exUpdateScrollFrame();
    if tabID == 2 then
      clDisplayWindow:Show();
      clTaskBar:Show();
      
      nbDisplayWindow:Hide();
      nbTaskBar:Hide();
    else
      clDisplayWindow:Hide();
      clTaskBar:Hide();
      
      nbDisplayWindow:Show();
      nbTaskBar:Show();
    end
  end
end


function Torta.gui:Initialize()
  
end

-- ====================================================================
--  Explorer listbox code
-- ====================================================================
function Torta.gui:CheckExplorerWindow()
  local numButtonsCanFit = floor((explorerWindow:GetHeight() - (20 + 2 + 2)) / EXPLORER_LISTBUTTON_HEIGHT);
  local numButtons = #self.exListButtons
  if numButtonsCanFit == numButtons then
    return;
  end
  
  if numButtonsCanFit > numButtons then
    self:CreateExplorerButtons(numButtonsCanFit - numButtons);
  else
    self:RemoveExplorerButtons(numButtons - numButtonsCanFit);
  end
end

function Torta.gui:CreateExplorerButtons(num)
  for i = 1, num do
    local id = #self.exListButtons + 1;
    local button = (self.exRecycleBin[id] or CreateFrame(
        "Button",
        nil,
        explorerWindow,
        "TortaExplorerButtonTemplate"));
    button:ClearAllPoints();
    button:SetPoint("TOPLEFT", self.exListButtons[id - 1], "BOTTOMLEFT");
    button:SetPoint("TOPRIGHT", self.exListButtons[id - 1], "BOTTOMRIGHT");
    button:SetID(id);
    button:Show();
    self.exListButtons[id] = button;
  end
  self:exUpdateScrollFrame();
end

function Torta.gui:RemoveExplorerButtons(num)
  for i = 1, num do
    local id = #self.exListButtons
    self.exRecycleBin[id] = self.exListButtons[id];
    self.exListButtons[id]:Hide();
    self.exListButtons[id] = nil;
  end
  self:exUpdateScrollFrame();
end
-- ====================================================================



function Torta.gui:clCheckButtons()
  local numButtonsCanFit = floor((clDisplayWindow:GetHeight() - (20 + 2 + 2)) / 32);
  local numButtons = #self.clListButtons
  if numButtonsCanFit == numButtons then
    return;
  end
  
  if numButtonsCanFit > numButtons then
    -- There's space for more buttons. Go make them.
    self:CreateCLButtons(numButtonsCanFit - numButtons);
  else
    -- There are too many buttons. Put some away.
    self:RemoveCLButtons(numButtons - numButtonsCanFit);
  end
end

function Torta.gui:CreateCLButtons(num)
  for i = 1, num do
    local id = #self.clListButtons + 1;
    local button = (self.clRecycleBin[id] or CreateFrame(
        "Button",
        nil,
        clDisplayWindow,
        "TortaChecklistTaskButton"));
    button:ClearAllPoints();
    button:SetPoint("TOPLEFT", self.clListButtons[id - 1], "BOTTOMLEFT");
    button:SetPoint("TOPRIGHT", self.clListButtons[id - 1], "BOTTOMRIGHT");
    button:SetID(id);
    button:Show();
    self.clListButtons[id] = button;
  end
  self:clUpdateScrollFrame();
end

function Torta.gui:RemoveCLButtons(num)
  for i = 1, num do
    local id = #self.clListButtons
    self.clRecycleBin[id] = self.clListButtons[id];
    self.clListButtons[id]:Hide();
    self.clListButtons[id] = nil;
  end
  self:clUpdateScrollFrame();
end


function Torta.gui:IsTaskVisible(id)
  for k,button in pairs(self.clListButtons) do
    if button:GetID() == id then
      return true;
    end
  end
  return false;
end


function Torta.gui:MakeTaskVisible(taskID)
  assert(type(taskID) == "number", "Assertion failed: method MakeTaskVisible requires a number parameter (taskID)");
  local scrollBar = getglobal(clScrollBar:GetName() .. "ScrollBar");
  local offset = FauxScrollFrame_GetOffset(clScrollBar);
  if taskID > self.clListButtons[1]:GetID() then
    for i = 1, 1000 do  -- just so we don't go into an infinite loop
      if self:IsTaskVisible(taskID) then
        break;
      end
      
      FauxScrollFrame_SetOffset(clScrollBar, offset);
      FauxScrollFrame_Update(clScrollBar, #Torta.cl3:GetLoadedList().text, #self.clListButtons, 32, nil, nil, nil, nil, nil, nil, true)
      self:clCheckButtons();
      self:clUpdateScrollFrame();
      offset = offset + 1
    end
    scrollBar:SetValue(floor(offset * 32 + 0.5));
    -- print(offset, scrollBar:GetValue());
  else
    for i = 1, 1000 do  -- just so we don't go into an infinite loop
      if self:IsTaskVisible(taskID) then
        break;
      end
      
      FauxScrollFrame_SetOffset(clScrollBar, offset);
      FauxScrollFrame_Update(clScrollBar, #Torta.cl3:GetLoadedList().text, #self.clListButtons, 32, nil, nil, nil, nil, nil, nil, true)
      self:clCheckButtons();
      self:clUpdateScrollFrame();
      offset = offset - 1
    end
    scrollBar:SetValue(floor(offset * 32 + 0.5));
    -- print(offset, scrollBar:GetValue());
  end
end

-- ====================================================================
--  Drag and drop functions
-- ====================================================================
function Torta.gui:clShowDropHandlers()
  for _, button in pairs(self.clListButtons) do
    button.topDropHandler:Show();
    button.bottomDropHandler:Show();
    UIFrameFadeOut(button.text, 0.3, button.text:GetAlpha(), 0.5);
    UIFrameFadeOut(button.check, 0.3, button.check:GetAlpha(), 0.5);
  end
end

function Torta.gui:clHideDropHandlers()
  for _, button in pairs(self.clListButtons) do
    button.topDropHandler:Hide();
    button.bottomDropHandler:Hide();
    UIFrameFadeIn(button.text, 0.3, button.text:GetAlpha(), 1);
    UIFrameFadeIn(button.check, 0.3, button.check:GetAlpha(), 1);
  end
end

function Torta.gui:clTaskDragStart(taskButton)
  PlaySound("INTERFACESOUND_CURSORGRABOBJECT");
  local text = taskButton.text:GetText();
  local stat = taskButton.check:GetChecked();
  TortaCLTaskDragThingy.text:SetText(text);
  TortaCLTaskDragThingy.check:SetChecked(stat);
  TortaCLTaskDragThingy:Show();
  UIFrameFadeIn(TortaCLTaskDragThingy, 0.25, 0, 1);
  self:clShowDropHandlers();
  self.dragSrcID = taskButton:GetID();
end

function Torta.gui:clTaskDragStop()
  if TortaCLTaskDragThingy:IsVisible() then
    TortaCLTaskDragThingy:Hide();
    PlaySound("INTERFACESOUND_CURSORDROPOBJECT");
    if GetMouseFocus() and GetMouseFocus().tortaDropType == "CLTASK" then
      Torta.cl3:MoveTask(self.dragSrcID, GetMouseFocus().destLoc);
      GetMouseFocus():Hide(); -- If this line isn't here, the drop handler under the mouse will fail to hide
    end
  end
  self:clHideDropHandlers();
end


function Torta.gui:clexShowDropHandlers()
  for _, button in pairs(self.exListButtons) do
    button.topDropHandler:Show();
    button.bottomDropHandler:Show();
    UIFrameFadeOut(button.text, 0.3, button.text:GetAlpha(), 0.5);
  end
end

function Torta.gui:clexHideDropHandlers()
  for _, button in pairs(self.exListButtons) do
    button.topDropHandler:Hide();
    button.bottomDropHandler:Hide();
    UIFrameFadeIn(button.text, 0.3, button.text:GetAlpha(), 1);
  end
end

function Torta.gui:clListDragStart(listButton)
  if self.canReorderLists then
    PlaySound("INTERFACESOUND_CURSORGRABOBJECT");
    local text = listButton.text:GetText();
    TortaCLListDragThingy.text:SetText(text);
    TortaCLListDragThingy:Show();
    UIFrameFadeIn(TortaCLListDragThingy, 0.25, 0, 1);
    self:clexShowDropHandlers();
    self.dragSrcID = listButton:GetID();
  end
end

function Torta.gui:clListDragStop()
  if TortaCLListDragThingy:IsVisible() then
    PlaySound("INTERFACESOUND_CURSORDROPOBJECT");
    TortaCLListDragThingy:Hide();
    if GetMouseFocus() and GetMouseFocus().tortaDropType == "CLLIST" then
      Torta.cl3:MoveList(self.dragSrcID, GetMouseFocus().destLoc);
      GetMouseFocus():Hide(); -- If this line isn't here, the drop handler under the mouse will fail to hide
    end
  end
  self:clexHideDropHandlers();
end
-- ====================================================================
-- End of drag and drop functions
-- ====================================================================

-- Shift-click to add item links to editbox. At last!
-- (Not yet complete though...)
hooksecurefunc("SetItemRef", function(link, text, button, ...)
  if strsub(link, 1, 4) == "item" and Torta.cl3:IsEditTaskDialogOpen() then
    editTaskEB:Insert(text);
  end
end)

-- ====================================================================
--  Misc. UI init code
-- ====================================================================
FauxScrollFrame_SetOffset(exScrollFrame, 0);
FauxScrollFrame_SetOffset(clScrollBar, 0);

mainWindow:SetMinResize(820, 450);
PanelTemplates_SetNumTabs(mainWindow, 3);
Torta.gui:OnTabClicked(1);

Torta:RegisterRevision("$Revision: 30 $");

-- ====================================================================
-- ====================================================================
-- Personal sandbox
-- Only for testing out SimpleHTML before using it in the addon
-- double-brackets have a space between them when commented out
-- to prevent those "nesting of [[...]] is deprecated" errors.
--[[
do
  local frame = CreateFrame("Frame");
  frame:SetPoint("CENTER", UIParent, "CENTER");
  frame:SetWidth(400);
  frame:SetHeight(300);
  
  local tex = frame:CreateTexture(nil, "BACKDROP");
  tex:SetTexture(0,0,1,0.5);
  tex:SetAllPoints(frame);
  
  local html = CreateFrame("SimpleHTML", nil, frame);
  html:SetParent(UIParent);
  html:SetText([ [
  <HTML>
  <BODY>
    <H1>
      Hello, World!
    </H1>
    <P>
      Once upon a time, in a faraway land... <BR /><BR /> ...And Linux pwned Windows. The End.
    </P>
  </BODY>
  </HTML>
  ] ]);
  
  --html:SetFont("Fonts\\FRIZQT__.TTF", 11);
  local gfnlFont = { GameFontNormalLarge:GetFont() };
  local gfhFont  = { GameFontHighlight:GetFont() };
  html:SetFont("H1", unpack(gfnlFont));
  html:SetFont("P", unpack(gfhFont));
  
  html:SetAllPoints(frame);
  frame:Show();
end
--]]

--[[
local f=CreateFrame("ScrollingMessageFrame","testaddon_Frame",UIParent); --creates the frame

f:SetWidth(200); f:SetHeight(20); --sets frame dimensions

f:SetPoint("LEFT",UIParent); --anchors the frame to the left edge of the screen

f:SetFontObject("GameFontNormal"); --tells the frame to copy the styling from fontobject "GameFontNormal"

f:SetJustifyH("LEFT"); --tells the frame to left-justify text

f:SetScript("OnHyperlinkClick", function(self, ...) ChatFrame_OnHyperlinkShow(...) end); --tells the frame to use the default hyperlink "OnClick" action

f:SetFading(false); --prevents the message frame from fading the message (so that the hyperlink will stay there)

f:AddMessage("|cffffd000|Henchant:2963|h[Tailoring: Bolt of Linen Cloth]|h|r"); --adds an example hyperlink
--]]