--[[
    Torta: A World of Warcraft addon that helps you keep track of things
    Copyright (C) 2008-2009 Lawrence Patrick Calulo (Quezacolt)

    Torta is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Torta is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Torta.  If not, see <http://www.gnu.org/licenses/>.
]]
-- ====================================================================
--  UIUpdate.lua
--  Contains code that refreshes on-screen data.
--  (Duh!)
--  Code is split up into numerous local functions for clarity
--  (*Hopefully* it's easier to understand this way...
-- ====================================================================

local widgets = Torta.gui.elements;

do  -- Checklist display function's very own scope
  local lineplusoffset;
  local listButton;
  
  local function updateListButtons(self)
    listButton.text:SetFormattedText("|cffffd000%i.)|r %s", lineplusoffset, self.listData.text[lineplusoffset]);
    listButton.check:SetChecked(self.listData.stat[lineplusoffset]);
    listButton:SetID(lineplusoffset);
    listButton.check:SetID(lineplusoffset);
    
    if lineplusoffset ~= Torta.cl3.openTask then
      listButton:UnlockHighlight();
    else
      listButton:LockHighlight();
    end
  end
  
  local function updateDropHandlers(self)
    local topText = "Insert above task " .. lineplusoffset;
    local btmText = "Insert below task " .. lineplusoffset;
    
    listButton.topDropHandler:SetFormattedText("%s - %s - %s", topText, topText, topText);
    listButton.topDropHandler.destLoc = lineplusoffset;
    listButton.bottomDropHandler:SetFormattedText("%s - %s - %s", btmText, btmText, btmText);
    listButton.bottomDropHandler.destLoc = lineplusoffset + 1;
  end
  
  local function updateDisplay(self)
      -- The last arg (true) makes the scroll bar visible at all times.
      FauxScrollFrame_Update(widgets.clScrollBar, #self.listData.text,
          #self.clListButtons, 32, nil, nil, nil, nil, nil, nil, true)
          
      if not widgets.clScrollBar:IsVisible() then
        widgets.clScrollBar:Show();
      end
      
      for line = 1, #self.clListButtons do
        lineplusoffset = line + FauxScrollFrame_GetOffset(widgets.clScrollBar);
        listButton = self.clListButtons[line];
        if lineplusoffset <= #self.listData.text then
          updateListButtons(self);
          updateDropHandlers(self);
          listButton:Show();
        else
          listButton:Hide();
        end
      end
      if #self.listData.text == 0 then
        widgets.noTasksText:Show();
        widgets.clScrollBar:Hide()
      else
        widgets.noTasksText:Hide();
        widgets.clScrollBar:Show();
      end
  end
  
-- ====================================================================
-- Checklist display function
-- ====================================================================
  function Torta.gui:clUpdateScrollFrame()
    self = self or Torta.gui;
    
    widgets.noTasksText:Hide();
    widgets.noLoadedListText:Hide();
    
    widgets.clDisplayWindow:Show();
    widgets.nbDisplayWindow:Hide();
    
    if self.listData then -- If a list is loaded
      widgets.noLoadedListText:Hide();
      widgets.cl3HeaderText:SetText(self.listData.meta.listName);
      updateDisplay(self);
    else -- If no list is loaded
      widgets.noLoadedListText:Show();
      widgets.clScrollBar:Hide();
      for line = 1, #self.clListButtons do
        self.clListButtons[line]:Hide();
      end
    end
    Torta.cl3:UpdateStats();
    self:UpdateLDB();
  end
  -- ====================================================================
  -- End of Checklist display function
  -- ====================================================================
end -- End of CL3 display function local scope





-- ====================================================================
--             * * * * * BEGIN EXPLORER SECTION * * * * *
-- ====================================================================
do
  -- ====================================================================
  -- Explorer display function
  -- This function is horribly long and complex at the moment.
  -- I am seriously disappointed at myself for writing this code. :(
  -- ====================================================================
  function Torta.gui:exUpdateScrollFrame()
    self = self or Torta.gui;
    local lineplusoffset;
    local button;
    widgets.explorerNoListsText:Hide();
    widgets.explorerNoNtbksText:Hide();
    
    if self.tab == 2 then -- if Torta Checklist is selected
      if #Torta.db.profile.lists > 0 then -- If we have lists
        local lists, indices;
        
        if Torta.cl3.isSearching then
          -- If we're searching, use the search results
          lists, indices = Torta.cl3:GetSearchMatches();
        else
          -- If not, use the lists in the DB
          lists = Torta.db.profile.lists;
        end
        
        FauxScrollFrame_Update(widgets.exScrollFrame, #lists, #self.exListButtons, 16, nil, nil, nil, nil, nil, nil, true)
        
        local loadedListID = Torta.db.profile.cfg.loadedList;
        for line = 1, #self.exListButtons do
          lineplusoffset = line + FauxScrollFrame_GetOffset(widgets.exScrollFrame);
          button = self.exListButtons[line];
          
          local limit;
          local searchMatches = Torta.cl3:GetSearchMatches();
          local searching = Torta.cl3.isSearching;
          
          if (lineplusoffset <= #lists) then -- If the current listbox button being updated has a corresponding list...
            if not lists[lineplusoffset] then
              return;
            end
            
            if not searching then
              listName = lists[lineplusoffset].meta.listName;
            else
              listName = lists[lineplusoffset];
            end
            
            button.text:SetText(listName);
            button.text.listName = listName;
            button:SetID(lineplusoffset);
            
            if not searching then
              if lineplusoffset ~= loadedListID then
                button:UnlockHighlight();
              else
                button:LockHighlight();
              end
            else
              if Torta.db.profile.cfg.loadedList ~= indices[lineplusoffset] then
                button:UnlockHighlight();
              else
                button:LockHighlight();
              end
            end
            
            button.topDropHandler.destLoc = lineplusoffset;
            button.bottomDropHandler.destLoc = lineplusoffset + 1;
            
            button:Show();
          else -- The listbox button being updated has no corresponding list.
            button:Hide();
          end
        end
      else -- If we don't have lists...
        widgets.exScrollFrame:Hide();
        widgets.explorerNoListsText:Show();
        for i = 1, #self.exListButtons do
          self.exListButtons[i]:Hide();
        end
      end
    else  -- if Torta Notebook is selected
      -- NYI
    end
  end
  -- ====================================================================
  --  End of retardedly long Explorer display function
  -- ====================================================================
end

